//+----------------------------------------------------------------------+
//|    Multiple Indicator EA                           mi_v5.mq4         |
//|                                              Copyright  2005        |
//|    Thanks to Starter for the order entry routines, and Fukinagashi.  |
//|    Special thanks to MrPip in his extensive help in the programming  |
//|    effort, correcting my errors, and teaching more about programming.|
//+----------------------------------------------------------------------+
#property copyright "Copyright  2005, Strategybuilderfx members"
#property link      "http:/strategybuilderfx.com/"
#include <stdlib.mqh>

extern bool Debug=true;                 // Change to true to allow print.

//+---------------------------------------------------+
//|Account functions                                  |
//+---------------------------------------------------+

extern bool AccountIsMini = false;       // Change to true if trading mini account.

//+---------------------------------------------------+
//|Money Management                                   |
//+---------------------------------------------------+

extern bool MoneyManagement = false;   // Change to false if you want to shutdown money management controls.
                                     // Lots = 1 will be in effect and only 1 lot will be open regardless of equity.
extern double Riskpercent = 10;     // Change to whatever percent of equity you wish to risk.
extern double DecreaseFactor = 3;   // Used to decrease lot size after a loss is experienced to protect equity.  Recommended not to change.
extern double StopLoss = 25;        // Maximum pips willing to lose per position.
extern double TrailingStop = 0;     // Change to whatever number of pips you wish to trail your position with.
extern int MaximumLosses = 10000;       // Maximum number of losses per day willing to experience.
extern double Margincutoff = 800;   // Expert will stop trading if equity level decreases to that level.
extern double Maxtrades = 100000;        // Total number of positions on all currency pairs. You can change this number as you wish.

//+---------------------------------------------------+
//|Profit controls                                    |
//+---------------------------------------------------+

extern int TakeProfit = 0;         // Maximum profit level achieved.    
extern double Stop = 0;             // Minimum profit level achieved and usually achieved target.
extern int Slippage = 10;           // Possible fix for not getting filled or closed    

//+---------------------------------------------------+
//|Indicator Variables                                |
//+---------------------------------------------------+

extern double MA1PeriodEntry=4;                   // Moving average period 1.
extern double MA2PeriodEntry=9;                   // Moving average period 2.
extern double MA3PeriodEntry=16;                   // Moving average period 3. 
extern double MA4PeriodExit=4;                    // Moving average period 4. Possibly use for exit condition.
extern double MA5PeriodExit=11;                   // Moving average period 5. Possibly use for exit condition.
extern double MA6PeriodExit=25;                   // Moving average period 6. Possibly use for exit condition.
extern bool TestMACrossMA1MA2ForEntry=false;      // Test for moving average cross of 2 moving averages.
extern bool TestMACrossMA1MA2MA3ForEntry=true;   // Test for moving average cross of 3 moving averages.
extern bool TestMACrossMA4MA5ForExit=true;       // Test for moving average cross.
extern bool TestMACrossMA4MA5MA6ForExit=false;    // Test for moving average cross.
extern bool TestRSICross=false;                   // Test for RSI cross of input value.
extern bool TestRSIAngleEntry=true;                   // Test for RSI angle for entry.
extern double RSICross=50;                        // Exit value for RSI cross.
extern double RSIPeriod=13;                       // RSI period 1.
extern bool TestCCICross=false;                   // Test for CCI cross of fast CCI input value.
extern bool TestCCIAngleEntry=true;                  // Test for CCI angle for entry.
extern double CCILong=100;                        // Value to check for entry long.
extern double CCIShort=-100;                      // Value to check for entry short.
extern double CCICross=0;                        // Exit value for CCI cross.
extern double CCIPeriod=34;                        // CCI value for fast period.
extern bool TestStochAngle=true;                 // Test if stochastics indicator is rising or falling.
extern double StochKPeriod=8;                     // Stochastic period %K.
extern double StochDPeriod=5;                     // Stochastic period %D.
extern double StochSlowing=5;                     // Stochastic slowing.
extern double StochHigh=75;                  // Tests for 
extern double StochLow=20;
extern bool TestBollingerWidth=true;         // Tests for standard deviation width.
extern double BollingerPeriod1=5;                 // Bollinger band period 1.
extern double BollingerStdDev1=3;                 // Bollinger band standard deviation period 1.
//extern bool TestBollingerHighExit=false;          // Tests for wide spread of Bollinger band during spikes that affect other exit conditions.
extern double BollingerWidthLow=10;               // Bollinger band measurement of standard deviation. Low is used for entry conditions.
//extern double BollingerWidthHigh=75;              // Bollinger band measurement of standard deviation. High is used for exit conditions.
extern double Lots = 1;                           // standard lot size. 
extern bool Turnon = true;                        // Turns expert on, change to false and no trades will take place even if expert is enabled.

//+---------------------------------------------------+
//|General controls                                   |
//+---------------------------------------------------+

string OrderText = "";
double lotMM;
int TradesInThisSymbol;
int cnt=0, total;
datetime LastTime;
double Sl;
double Tr;
int ticket;
int trstop = 0;

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+

int init()
  {
   int handle, err;
   
    string filename=Symbol()+TimeToStr(CurTime(),TIME_DATE) + ".txt";
    
        
//---- 
    if (Debug)
    {
      GlobalVariableSet("MyHandle",0);
      handle = FileOpen(filename,FILE_CSV|FILE_WRITE,"\t");
//    if (Debug) Print ("Handle: ",handle);
      if (!GlobalVariableCheck("MyHandle"))
      {
         err = GetLastError();
         Print("Error creating Global Variable MyHandle: (" + err + ") " + ErrorDescription(err)); 
      }
      GlobalVariableSet("MyHandle",handle); 
    }
//----
   return(0);
  }
  
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+

int deinit()
  {
     int handle;
 
    if (Debug)
    {
      handle = GlobalVariableGet("MyHandle");
      FileFlush(handle);
      FileClose(handle);
      GlobalVariableDel("MyHandle");  
    }
   return(0);
  }

//+------------------------------------------------------------------+
//| Write - writes string to debug file                              |
//+------------------------------------------------------------------+

int Write(string str)
{
   int handle;
   
   handle = GlobalVariableGet("MyHandle");
   FileWrite(handle,str,"\r\n"); 
}
  

//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+

double LotsOptimized(int Mnr)
  {
   double lot=Lots;
   int    orders=HistoryTotal();     // history orders total
   int    losses=0;                  // number of losses orders without a break
   int    tolosses=0;
//---- select lot size
   lot=NormalizeDouble(MathFloor(AccountFreeMargin()*Riskpercent/10000)/10,1);
   
//+------------------------------------------------------------------+  
//|  Calculate number of losses orders without a break               |
//+------------------------------------------------------------------+

   if(DecreaseFactor>0)
     {
      for(int i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
         if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL || OrderMagicNumber()!=Mnr) continue;
         //----
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++;
        }
      for(i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
         if(TimeDay(OrderCloseTime()) != TimeDay(CurTime())) continue;
         //----
         if(OrderProfit()<0) tolosses++;
        }
        if (tolosses >= MaximumLosses) trstop=1;
      if(losses>1) lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
     }
     
  // Check if mini or standard Account

  if(AccountIsMini)
  { 
      lot = MathFloor(lot);
      lot = lot / 10;
// Use at least 1 mini lot
      if(lot<0.1) lot=0.1;
  }else
  {
    if (lot < 1.0) lot = 1.0;
    if (lot > 100) lot = 100;
  }         

   return(lot);
  } 
  
//+-------------------------------------------------------------------------------------------+
//| Exit Strategies                                                                           |
//|      One or more exit strategies can be choosen to exit                                   |
//|       the trade. If Test for exit condition is set to false                               |
//|       for any exit condition, that condition will be set to                               |
//|       1 and not affect any combinations of other exit strategies.                         |
//|       Add to ExitConditionsLong to compare with ExitCntLong                               |
//|       for exit of trade.                                                                  |
//|       Add to ExitConditionsShort to compare with ExitCntShort                             |
//|       for exit of trade.                                                                  |
//+-------------------------------------------------------------------------------------------+
  
  
//+--------------------------------------------------------------------+
//| Moving average cross for exit.                                     |
//|   If Test MA Cross is false, exit value will equal 1 for exitcnt   |
//|      Long and 1 for exitcnt Short to allow other exit strategies   |
//|      to function.                                                  |
//|   If fast MA crosses below slow MA on BUY, add 1 to exitcntLong.   |
//|   If fast MA crosses above slow MA on SELL, add 1 to exitcntShort. |
//|                                                                    |
//+--------------------------------------------------------------------+
bool CheckMACross(string type, int which)
{
   double MA4, MA5, MA6;

//+------------------------------------------------------------------+
//| Calculate moving averages                                        |
//+------------------------------------------------------------------+

     MA4=iMA(NULL, 0, MA4PeriodExit, 0, MODE_LWMA, PRICE_WEIGHTED, 1);
     MA5=iMA(NULL, 0, MA5PeriodExit, 0, MODE_LWMA, PRICE_WEIGHTED, 1);
     MA6=iMA(NULL, 0, MA6PeriodExit, 0, MODE_LWMA, PRICE_WEIGHTED, 1);
    
     if(type=="BUY")
     {
         if (which == 2)
         {
            if (MA4<MA5)
            {
               if(Debug)
               {
                  Print("Single MA Cross met for exit long");
                  Write("Single MA Cross met for exit long");
               }
               return(true);
             }
          }

          if (which == 3)
          {
            if (MA4<MA5<MA6)
             {
               if(Debug)
               {
                  Print("Two MA Cross met for exit long");
                  Write("Two MA Cross met for exit long");
               }
               return(true);
             }
          }
     }
     if (type=="SELL")
     {
          if (which == 2)
          {
            if (MA4>MA5)
            {
               if(Debug)
               {
                  Print("Single MA Cross met for exit short");
                  Write("Single MA Cross met for exit short");
               }
               return(true);
             }
          }
          if (which == 3)
          {
            if (MA4>MA5>MA6)
             {
               if(Debug)
               {
                  Print("Two MA Cross met for exit short");
                  Write("Two MA Cross met for exit short");
               }
               return(true);
             }
          }
     }
     
     return(false);
}

//+------------------------------------------------------------------+
//| RSI cross for exit.                                              |
//|                                                                  |
//|   If RSI crosses below exit value on BUY, returns true.          |
//|   If RSI crosses above exit value on SELL, returns true.         | 
//|                                                                  |                                                           
//+------------------------------------------------------------------+
bool CheckRSICross(string type)
{
   double RSI_Val;
   
//+------------------------------------------------------------------+
//| Calculate RSI values                                             |
//+------------------------------------------------------------------+
   
   RSI_Val=iRSI(NULL, 0, RSIPeriod, PRICE_WEIGHTED, 1);
   
   if (type == "BUY")
   {
      if (RSI_Val < RSICross)
      {
          if(Debug)
          {
             Print("RSI Cross met for exit long");
             Write("RSI Cross met for exit long");
          }
          return(true);
       }
   }
   if (type == "SELL")
   {
       if (RSI_Val > RSICross)
       {
          if(Debug)
          {
             Print("RSI Cross met for exit short");
             Write("RSI Cross met for exit short");
          }
          return(true);
       }
   }
   return(false);
}

//+------------------------------------------------------------------+
//| CCI cross for exit.                                              |
//|                                                                  |
//|   If CCI crosses below exit value on BUY, returns true.          |
//|   If CCI crosses above exit value on SELL, returns true.         | 
//|                                                                  |                                             
//+------------------------------------------------------------------+

bool CheckCCICross(string type)
{ 
   double CCI_Val;
   
//+------------------------------------------------------------------+
//| Calculate CCI values                                             |
//+------------------------------------------------------------------+

     CCI_Val=iCCI(NULL, 0, CCIPeriod, PRICE_WEIGHTED, 1);
     if (type == "BUY")
     {
        if (CCI_Val < CCICross)
        {
          if(Debug)
          {
             Print("CCI named value Cross met for exit long");
             Write("CCI named value Cross met for exit long");
          }
          return(true);
        }
     }
     if (type == "SELL")
     {   
        if (CCI_Val > CCICross)
        {
           if(Debug)
           {
              Print("CCI named value Cross met for exit short");
              Write("CCI named value Cross met for exit short");
           }
           return(true);
         }
      }
      return(false);       
}

//+------------------------------------------------------------------+
//| Bollinger band extreme width.                                    |
//|   If Test Bollinger Width High is true, the next indicator       |
//|      calling for exit will execute exit.                         |
//|      It will add +3 to exitcntLong.                              |
//|      It will add -3 to exitcntShort.                             |
//|   If Test is false, value will be added to exitcnt (+1,-1) to    |
//|      allow exit if other indicators call for exit.               |
//|                                                                  |
//|                                                                  |
//+------------------------------------------------------------------+
//bool CheckBB(string type)
//{
//   double BB1Lower,BB1LowerPrevious;
//   double BB1Upper,BB1UpperPrevious;
//   double BB1Width,BB1WidthPrevious;
//}

//bool CheckStoch(string type)
//{
//  double Stoch1Main,Stoch1MainPrevious;
//  double Stoch1Signal,Stoch1SignalPrevious;
   
//+------------------------------------------------------------------+
//| Calculate Stochastic value                                       |
//+------------------------------------------------------------------+
  
//  Stoch1Main=iStochastic(NULL, 0, StochKPeriod, StochDPeriod, StochSlowing, MODE_LWMA, 0, MODE_MAIN, 1);
//  Stoch1MainPrevious=iStochastic(NULL, 0, StochKPeriod, StochDPeriod, StochSlowing, MODE_LWMA, 0, MODE_MAIN, 2);
//  Stoch1Signal=iStochastic(NULL, 0, StochKPeriod, StochDPeriod, StochSlowing, MODE_LWMA, 0, MODE_SIGNAL, 1);
//  Stoch1SignalPrevious=iStochastic(NULL, 0, StochKPeriod, StochDPeriod, StochSlowing, MODE_LWMA, 0, MODE_SIGNAL, 2);
//}


//+-------------------------------------------------------------------------------------------+
//| CheckExitCondition                                                                        |
//| Check if any exit condition is met                                                        |
//+-------------------------------------------------------------------------------------------+

bool CheckExitCondition(string TradeType)
{
   bool YesClose;
   int ExitCntLong=0;   // Totals the number of long exit conditions met to be compared to total exit strategies testing for.
   int ExitCntShort=0;  // Totals the number of short exit conditions met to be compared to total exit strategies testing for.
   int ExitConditionsLong=0;   // Totals the number of trade exit strategies set to true.
   int ExitConditionsShort=0;  // Totals the number of trade exit strategies set to true.
   
   YesClose = false;
   
   if (TradeType == "BUY")
   {
   
// Check for 2 moving averages crossing.
      
         if (TestMACrossMA4MA5ForExit)
         {
           ExitConditionsLong++;
           if (CheckMACross("BUY",2)) ExitCntLong++;
         }
    
//Check for 3 moving averages crossing.

         if (TestMACrossMA4MA5MA6ForExit)
         {
           ExitConditionsLong++;
           if (CheckMACross("BUY",3)) ExitCntLong++;
         }
   
//Check for RSI crossing.

         if (TestRSICross)
         {
           ExitConditionsLong++;
           if (CheckRSICross("BUY")) ExitCntLong++;
         }


// Check for CCI Crossing

        if (TestCCICross)
        {
           ExitConditionsLong++;
           if (CheckCCICross("BUY")) ExitCntLong++;
        }
         

//        if (TestStochAngle)                 // Test if stochastics indicator is rising or falling.
//        {
//           ExitConditionsLong++;
//           if (CheckStoch("BUY")) ExitCntLong++;
//        }
        
// Add other exit trategies here for long


        if(ExitCntLong >= ExitConditionsLong) YesClose = true;
     }
     if (TradeType == "SELL")
     {
         if (TestMACrossMA4MA5ForExit)
         {
           ExitConditionsShort++;
           if (CheckMACross("SELL",2)) ExitCntShort++;
         }
    
//Check for 3 moving averages crossing.

         if (TestMACrossMA4MA5MA6ForExit)
         {
           ExitConditionsShort++;
           if (CheckMACross("SELL",3)) ExitCntShort++;
         }
         
//Check for RSI crossing.

         if (TestRSICross)
         {
           ExitConditionsShort++;
           if (CheckRSICross("SELL")) ExitCntShort++;
         }
         
// Check for CCI Crossing

        if (TestCCICross)
        {
           ExitConditionsShort++;
           if (CheckCCICross("SELL")) ExitCntShort++;
        }
        
// Check for Stochastic Condition
        
//        if (TestStochAngle)                 // Test if stochastics indicator is rising or falling.
//        {
//           ExitConditionsShort++;
//           if (CheckStoch("SELL")) ExitCntShort++;
//        }
        
// Add other exit trategies here for short
         
        if(ExitCntShort >= ExitConditionsShort) YesClose = true;
     }
     
//+------------------------------------------------------------------+
//| Friday Exits                                                     |
//+------------------------------------------------------------------+

     if(DayOfWeek()==5 && Hour()>=20) YesClose = true;

     return (YesClose);
}

//+-------------------------------------------------------------------------------------------+
//| Entrance Strategies                                                                       |
//|      One or more entrance strategies can be choosen to enter                              |
//|       the trade. If Test for entrance condition is set to false                           |
//|       for any entrance condition, that condition will effect                              |
//|       any combinations of other entrance strategies.                                      |
//|      If no entrance strategy is choosen, an error will come back                          |
//|       NO ENTRANCE CONDITION CHOOSEN, TRADING NOT ALLOWED                                  |
//+-------------------------------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| Moving average cross for entrance.                               |
//|   If Test MA Cross is false, exit value will equal 1 for EntryCnt|
//|      Long and 1 for EntryCnt Short to allow other exit           |
//|      strategies to function.                                     |
//|   If fast MA crosses below slow MA on BUY, returns 1.            |
//|   If fast MA crosses above slow MA on SELL, returns 1.           |
//|                                                                  |
//+------------------------------------------------------------------+


bool CheckMACrossEntry(string type, int which)
{
   double MA1, MA1Previous;
   double MA2, MA2Previous;
   double MA3, MA3Previous;

  MA1=iMA(NULL, 0, MA1PeriodEntry, 0, MODE_EMA, PRICE_WEIGHTED, 1);
  MA1Previous=iMA(NULL, 0, MA1PeriodEntry, 0, MODE_EMA, PRICE_WEIGHTED, 2);
  MA2=iMA(NULL, 0, MA2PeriodEntry, 0, MODE_LWMA, PRICE_WEIGHTED, 1);
  MA2Previous=iMA(NULL, 0, MA2PeriodEntry, 0, MODE_LWMA, PRICE_WEIGHTED, 2);
  MA3=iMA(NULL, 0, MA3PeriodEntry, 0, MODE_LWMA, PRICE_WEIGHTED, 1);
  MA3Previous=iMA(NULL, 0, MA3PeriodEntry, 0, MODE_LWMA, PRICE_WEIGHTED, 2);  

  if (which == 2)
  {
   if (type == "BUY")
   {
      if (MA1>MA2)
      {
         if (Debug)
         {
           Print("Single MA Cross met for entrance long");
           Write("Single MA Cross met for entrance long");
         }
         return(true);
      }
   }
   if (type == "SELL")
   {
     if (MA1<MA2)
     {
      if (Debug)
      {
         Print("Single MA Cross met for entrance short");
         Write("Single MA Cross met for entrance short");
      }      
      return(true);
     }
   }
  }   
     
  if (which == 3)
  {
    if(type == "BUY")
    {
      if ((MA1>MA2) && (MA1>MA3))
      {
        if (Debug)
        {
         Print("MA1>MA2 & MA1>MA3 met for entrance long");
         Write("MA1>MA2 & MA1>MA3 met for entrance long");
        }
        return(true);
      }
    }
    if (type == "SELL")
    {   
      if ((MA1<MA2) && (MA1<MA3))
      {
        if (Debug)
        {
          Print("MA1>MA2 & MA1>MA3 met for entrance short");
          Write("MA1>MA2 & MA1>MA3 met for entrance short");
        }      
        return(true);
      }
    }   
   }
   return (false);
}
   

//+------------------------------------------------------------------+
//| RSI cross for entrance.                                          |
//|   If Test RSI Cross is true, exitcnt value will be increased     |
//|      when exit condition is met.                                 |                                       
//|   If RSI crosses below exit value on BUY, returns 1.             |
//|   If RSI crosses above exit value on SELL, returns 1.            | 
//|                                                                  |                                                           
//+------------------------------------------------------------------+
bool CheckRSIAngleEntry(string type)
{
   double RSI1, RSI1Previous, RSI1Previous2;

//+------------------------------------------------------------------+
//| Calculate RSI values                                             |
//+------------------------------------------------------------------+
  
  RSI1=iRSI(NULL, 0, RSIPeriod, PRICE_WEIGHTED, 1);
  RSI1Previous=iRSI(NULL, 0, RSIPeriod, PRICE_WEIGHTED, 2);
  RSI1Previous2=iRSI(NULL, 0, RSIPeriod, PRICE_WEIGHTED, 3);

  if (type == "BUY")
  {
    if (RSI1>RSI1Previous)
    {
      if (Debug)
      {
         Print("RSI angle met for entrance long");
         Write("RSI angle met for entrance long");
      }
      return(true);
    }
  }
  if (type == "SELL")
  {   
    if (RSI1<RSI1Previous)
    {
      if (Debug)
      {
         Print("RSI angle met for entrance short");
         Write("RSI angle met for entrance short");
      }
      return(true);
    }   
   }
   return (false);
}

//+------------------------------------------------------------------+
//| CCI cross for entrance.                                          |
//|   If Test CCI Cross is true, exitcnt value will be increased     |
//|      when exit condition is met.                                 |                                       
//|                                                                  |
//|   If Test is false, value will be added to exitcnt to allow exit |
//|      if other indicators call for exit.                          |
//|   If CCI crosses below exit value on BUY, returns 1.             |
//|   If CCI crosses above exit value on SELL, returns 1.            | 
//|                                                                  |                                             
//+------------------------------------------------------------------+

bool CheckCCIAngleEntry(string type)
{
   double CCI1, CCI1Previous, CCI1Previous2;

//+------------------------------------------------------------------+
//| Calculate CCI values                                             |
//+------------------------------------------------------------------+

  CCI1=iCCI(NULL, 0, CCIPeriod, PRICE_WEIGHTED, 1);
  CCI1Previous=iCCI(NULL, 0, CCIPeriod, PRICE_WEIGHTED, 2);
  CCI1Previous2=iCCI(NULL, 0, CCIPeriod, PRICE_WEIGHTED, 3);


  if (type == "BUY")
  {
      if (((CCI1>CCI1Previous) && (CCI1Previous>CCI1Previous2)) || ((CCI1>CCILong) && (CCI1Previous>CCILong)))
      {
        if (Debug)
        {
         Print("CCI up angle met for entrance long");
         Write("CCI up angle met for entrance long");
        }
        return(true);
      }
   }
   if (type == "SELL")
   {   
     if (((CCI1<CCI1Previous) && (CCI1Previous<CCI1Previous2)) || ((CCI1<CCIShort) && (CCI1Previous<CCIShort)))
      {
       if (Debug)
       {
         Print("CCI down angle met for entrance short");
         Write("CCI down angle met for entrance short");
       }
       return(true);
     }   
   }
   return (false);
}


//+------------------------------------------------------------------+
//| Bollinger band extreme width.                                    |
//|   If Test Bollinger Width High is true, the next indicator       |
//|      calling for exit will execute exit.                         |
//|      It will add +3 to exitcntLong.                              |
//|      It will add -3 to exitcntShort.                             |
//|   If Test is false, value will be added to exitcnt (+1,-1) to    |
//|      allow exit if other indicators call for exit.               |
//|                                                                  |
//|                                                                  |
//+------------------------------------------------------------------+
bool CheckBBEntry(string type)
{
   double BB1Lower,BB1LowerPrevious,BB1LowerPrevious2;
   double BB1Upper,BB1UpperPrevious,BB1UpperPrevious2;
   double BB1Width,BB1WidthPrevious,BB1WidthPrevious2;
   double a,b,c;
   double bb1Width;
   


//+------------------------------------------------------------------+
//| Calculate Bollinger Band width                                   |
//+------------------------------------------------------------------+
  
  BB1Lower=iBands(NULL, 0, BollingerPeriod1, BollingerStdDev1, 0, PRICE_CLOSE, MODE_LOWER, 1);
  BB1LowerPrevious=iBands(NULL, 0, BollingerPeriod1, BollingerStdDev1, 0, PRICE_CLOSE, MODE_LOWER, 2);
  BB1LowerPrevious2=iBands(NULL, 0, BollingerPeriod1, BollingerStdDev1, 0, PRICE_CLOSE, MODE_LOWER, 3);
  BB1Upper=iBands(NULL, 0, BollingerPeriod1, BollingerStdDev1, 0, PRICE_CLOSE, MODE_UPPER, 1);
  BB1UpperPrevious=iBands(NULL, 0, BollingerPeriod1, BollingerStdDev1, 0, PRICE_CLOSE, MODE_UPPER, 2);
  BB1UpperPrevious2=iBands(NULL, 0, BollingerPeriod1, BollingerStdDev1, 0, PRICE_CLOSE, MODE_UPPER, 3);
  BB1Width=BB1Upper-BB1Lower;
  BB1WidthPrevious=BB1UpperPrevious-BB1LowerPrevious;
  BB1WidthPrevious2=BB1UpperPrevious2-BB1LowerPrevious2;

//  double GetBBWidthLimit()
//   {
//    double bbWidth;
//    bbWidth=BollingerWidthLow/10000;  //This takes extern statement and converts for comparison to bollinger value.
//    if (StringSubstr(Symbol(),3,3)=="JPY")
//       bbWidth=bbWidth*100;
//        return(bbWidth);
//    }

   if (StringSubstr(Symbol(),3,3)=="JPY")
      {
      bb1Width=BB1Width*100;
      }
      else
      {
      bb1Width=BB1Width*10000;
      }
  
 Print(Symbol()+" Bollinger width "+bb1Width);
 Write(Symbol()+" Bollinger width "+bb1Width);
// Print(Symbol()+" Bollinger width"+bbWidth);
// Write(Symbol()+" Bollinger width"+bbWidth);
Comment("Bollinger Width=",bb1Width);
 
   if (type == "BUY") 
      {
       if (bb1Width > BollingerWidthLow)
       {
         if (Debug)
         {
            Print("Bollinger band width greater than narrow limit, allow trade.");
            Write("Bollinger band width greater than narrow limit, allow trade.");
         }
         return(true);
       }   
      }
   if (type == "SELL")
      {
       if (bb1Width > BollingerWidthLow)
       {
         if (Debug)
         {
            Print("Bollinger band width greater than narrow limit, allow trade.");
            Write("Bollinger band width greater than narrow limit, allow trade.");
         }
         return(true);
       }   
      }
      return(false);
}

bool CheckStochEntry(string type)
{
   double Stoch1Main,Stoch1MainPrevious;
   double Stoch1Signal,Stoch1SignalPrevious;

//+------------------------------------------------------------------+
//| Calculate Stochastic value                                       |
//+------------------------------------------------------------------+
  
  Stoch1Main=iStochastic(NULL, 0, StochKPeriod, StochDPeriod, StochSlowing, MODE_LWMA, 0, MODE_MAIN, 1);
  Stoch1MainPrevious=iStochastic(NULL, 0, StochKPeriod, StochDPeriod, StochSlowing, MODE_LWMA, 0, MODE_MAIN, 2);
  Stoch1Signal=iStochastic(NULL, 0, StochKPeriod, StochDPeriod, StochSlowing, MODE_LWMA, 0, MODE_SIGNAL, 1);
  Stoch1SignalPrevious=iStochastic(NULL, 0, StochKPeriod, StochDPeriod, StochSlowing, MODE_LWMA, 0, MODE_SIGNAL, 2);

  if (type == "BUY")
  {
    if ((Stoch1Main>Stoch1MainPrevious) && (Stoch1Main>Stoch1Signal))// || ((Stoch1Main>StochHigh) && (Stoch1MainPrevious>StochHigh)))
    {
      if (Debug)
      {
         Print("Stochastic up angle met for entrance long " +Stoch1Main);
         Write("Stochastic up angle met for entrance long");
      }
         return(true);
    }
  }
  if (type == "SELL")
  {   
    if ((Stoch1Main<Stoch1MainPrevious) && (Stoch1Main>Stoch1Signal))// || ((Stoch1Main<StochLow) && (Stoch1MainPrevious<StochLow)))
    {
      if (Debug)
      {
         Print("Stochastic down angle met for entrance short " +Stoch1Main);
         Write("Stochastic down angle met for entrance short");
      }
      return(true);
    }   
   }
   return (false);
}

   
//+-------------------------------------------------------------------------------------------+
//| CheckEntryCondition                                                                       |
//| Check if any entry condition is met                                                       |
//+-------------------------------------------------------------------------------------------+

bool CheckEntryCondition(string TradeType)
{
   bool YesTrade;
   int EntryCntLong=0;    // Totals the number of long entry conditions met to be compared to total entry strategies testing for.
   int EntryCntShort=0;   // Totals the number of short entry conditions met to be compared to total entry strategies testing for.
   int EntryConditionsLong=0;  // Totals the number of trade exit strategies set to true.
   int EntryConditionsShort=0; // Totals the number of trade exit strategies set to true.
   
   YesTrade = false;
   
   if (TradeType == "BUY")
   {
   
// Check for 2 moving averages crossing.
      
         if (TestMACrossMA1MA2ForEntry)
         {
           EntryConditionsLong++;
           if (CheckMACrossEntry("BUY",2)) EntryCntLong++;
         }
    
//Check for 3 moving averages crossing.

         if (TestMACrossMA1MA2MA3ForEntry)
         {
           EntryConditionsLong++;
           if (CheckMACrossEntry("BUY",3)) EntryCntLong++;
         }
   
//Check for RSI angle.

         if (TestRSIAngleEntry)
         {
           EntryConditionsLong++;
           if (CheckRSIAngleEntry("BUY")) EntryCntLong++;
         }


// Check for CCI Crossing

        if (TestCCIAngleEntry)
        {
           EntryConditionsLong++;
           if (CheckCCIAngleEntry("BUY")) EntryCntLong++;
        }
         

        if (TestStochAngle)                 // Test if stochastics indicator is rising or falling.
        {
           EntryConditionsLong++;
           if (CheckStochEntry("BUY")) EntryCntLong++;
        }
        
// Check for Bollinger band width to allow trading

         if (TestBollingerWidth)            // Test width of Bollinger band for times of narrow channeling. 
         {
            EntryConditionsLong++;
            if (CheckBBEntry("BUY")) EntryCntLong++;
         }
        
// Add other entrance strategies here for long

        if(EntryCntLong >= EntryConditionsLong) YesTrade = true;
     }
     if (TradeType == "SELL")
     {
         if (TestMACrossMA1MA2ForEntry)
         {
           EntryConditionsShort++;
           if (CheckMACrossEntry("SELL",2)) EntryCntShort++;
         }
    
//Check for 3 moving averages crossing.

         if (TestMACrossMA1MA2MA3ForEntry)
         {
           EntryConditionsShort++;
           if (CheckMACrossEntry("SELL",3)) EntryCntShort++;
         }
         
//Check for RSI crossing.

         if (TestRSIAngleEntry)
         {
           EntryConditionsShort++;
           if (CheckRSIAngleEntry("SELL")) EntryCntShort++;
         }
// Check for CCI Crossing

        if (TestCCIAngleEntry)
        {
           EntryConditionsShort++;
           if (CheckCCIAngleEntry("SELL")) EntryCntShort++;
        }
        
        if (TestStochAngle)                 // Test if stochastics indicator is rising or falling.
        {
           EntryConditionsShort++;
           if (CheckStochEntry("SELL")) EntryCntShort++;
        }
        
// Check for Bollinger band width to allow trading

         if (TestBollingerWidth)            // Test width of Bollinger band for times of narrow channeling. 
         {
            EntryConditionsShort++;
            if (CheckBBEntry("SELL")) EntryCntShort++;
         }
        
// Add other entrance strategies here for short
         
        if(EntryCntShort >= EntryConditionsShort) YesTrade = true;
     }
     

     return (YesTrade);
}
          
          

//+-------------------------------------------------------------------------------------------+
//| expert start function                                                                     |
//+-------------------------------------------------------------------------------------------+

int start()
  {
//---- 
      
   double Min_OrderPrice;
   bool ExitBuy, ExitSell;
   int donttrade, allexit, err;
   
   trstop = 0;

	int MagicNumber = 3000 + func_Symbol2Val(Symbol())*100 + func_TimeFrame_Const2Val(Period()); 

   string setup="mi_v5c " + Symbol() + "_" + func_TimeFrame_Val2String(func_TimeFrame_Const2Val(Period()));
   
//|      If no exit strategy is choosen, an error will come back     |
//|       NO EXIT CONDITION CHOOSEN, TRADING NOT ALLOWED             |

   if ( !(TestMACrossMA4MA5ForExit || TestMACrossMA4MA5MA6ForExit || TestRSICross || TestCCICross))
   {
     Alert("NO EXIT CONDITION CHOSEN, TRADING NOT ALLOWED");
     return(0);
   }

   donttrade = 0;
   
//+------------------------------------------------------------------+
//| Friday Exits                                                     |
//+------------------------------------------------------------------+

   if(DayOfWeek()==5 && Hour()>=18) donttrade=1;

   
//+------------------------------------------------------------------+
//| Open Position Controls                                           |
//+------------------------------------------------------------------+

   ExitBuy = CheckExitCondition("BUY");
   ExitSell = CheckExitCondition("SELL");
   HandleOpenPositions(MagicNumber, ExitBuy, ExitSell);

// Check if any open positions were not closed

   TradesInThisSymbol = CheckOpenPositions(MagicNumber);
     


if(AccountFreeMargin() < Margincutoff) {
   return(0);}
if(TradesInThisSymbol > Maxtrades) {
   return(0);}
if(CurTime() < LastTime) {
   return(0);}
   
// Moved money management here because closed trades change account balance

if(MoneyManagement)
{
  lotMM = LotsOptimized(MagicNumber);
}
else {
  lotMM = Lots; // Change mm to 0 if you want the Lots parameter to be in effect
}

//+------------------------------------------------------------------+
//| New Position Controls                                            |
//+------------------------------------------------------------------+
     
  OrderText = ""; //Must be blank before going into the main section

	if (CheckEntryCondition("BUY") && !CheckExitCondition("BUY") && donttrade == 0)
	{
		OrderText = "BUY";
		if (StopLoss>0) {
		 Sl = Ask-StopLoss*Point;
		} else {
		 Sl=0;
		}
		if (TakeProfit == 0) 
		    Tr = 0;
		else
		    Tr = Ask+TakeProfit*Point;
	}

	if (CheckEntryCondition("SELL") && !CheckExitCondition("SELL") && donttrade == 0)
	{
		OrderText = "SELL";
		if (StopLoss>0) {
		 Sl = Bid+StopLoss*Point;
		} else {
		 Sl = 0;
		}
		if (TakeProfit == 0) 
		    Tr = 0;
		else
		    Tr = Bid-TakeProfit*Point;
	}
   if(OrderText != "" && trstop == 0 && TradesInThisSymbol <= 0 && Turnon) 
   {

	   LastTime = CurTime();
           if(OrderText == "BUY")
           {
               Min_OrderPrice=MinOrderPrice(OP_BUY, MagicNumber);
               if (Min_OrderPrice>0 && Min_OrderPrice<=Ask*1.05) {
                  Print("Buy too expensive => MinOrderPrice= " + Min_OrderPrice + "  Ask=" + Ask);
               } else {
                  Print(Symbol() + " OP_BUY " + lotMM + " " + Ask + " " + Sl + " " + Tr);
	              ticket = OrderSendExtended(Symbol(),OP_BUY,lotMM,Ask,3,Sl,Tr,setup,MagicNumber,0,Green);
                  if (!IsTesting()) PlaySound("expert.wav");
	              LastTime += 12;
                 if(ticket<=0) {
                     err = GetLastError();
                     Print("Error opening BUY order [" + setup + "]: (" + err + ") " + ErrorDescription(err)); 
                     if (!IsTesting()) PlaySound("alert2.wav");
                 }
                 return(0);
               }
           }
            else if(OrderText == "SELL")
            {
               Min_OrderPrice=MinOrderPrice(OP_SELL, MagicNumber);
         
               if (Min_OrderPrice>0 && Min_OrderPrice>=Bid) {
                  Print("Sell too expensive MinOrderPrice= " + Min_OrderPrice + "  Bid=" + Bid);
               } else {
                  Print(Symbol() + " OP_SELL " + lotMM + " " + Bid + " " + Sl + " " + Tr);
                  ticket = OrderSendExtended(Symbol(),OP_SELL,lotMM,Bid,3,Sl,Tr,setup,MagicNumber,0,Red);
                  if (!IsTesting()) PlaySound("expert.wav");
                  LastTime += 12;
                 if(ticket<=0) {
                     err = GetLastError();
                     Print("Error opening Sell order [" + setup + "]: (" + err + ") " + ErrorDescription(err)); 
                     if (!IsTesting()) PlaySound("alert2.wav");
                 }
                  return(0);
               }
            }
      return(0);
   }
//----
   return(0);
  }

//+------------------------------------------------------------------+
//| Check Open Position Controls                                     |
//+------------------------------------------------------------------+
  
int CheckOpenPositions(int MagicNumbers)
{
   int cnt, total, NumPositions;
   int NumBuyTrades, NumSellTrades;   // Number of buy and sell trades in this symbol
   
   NumBuyTrades = 0;
   NumSellTrades = 0;
   total=OrdersTotal();
   for(cnt=0;cnt<total;cnt++)
     {
      if ( OrderSelect (cnt, SELECT_BY_POS) == false )  continue;
      if ( OrderSymbol() != Symbol() || OrderMagicNumber() != MagicNumbers)  continue;
      
      if((OrderType() == OP_BUY || OrderType() == OP_BUYSTOP) && (OrderSymbol()==Symbol()))
          {
             NumBuyTrades++;
          }
             
      if((OrderType() == OP_SELL || OrderType() == OP_SELLSTOP) && (OrderSymbol()==Symbol()))
          {
             NumSellTrades++;
          }
             
     }
     NumPositions = NumBuyTrades + NumSellTrades;
     return (NumPositions);
  }


//+------------------------------------------------------------------+
//| Handle Open Positions                                            |
//| Check if any open positions need to be closed or modified        |
//| Three attempts are made to close or modify                       |
//+------------------------------------------------------------------+
int HandleOpenPositions(int MagicNum, bool BuyExit, bool SellExit)
{
   int cnt, total;
   int CloseCnt, err;
   
   total=OrdersTotal();
   for(cnt=0;cnt<total;cnt++)
     {
      if ( OrderSelect (cnt, SELECT_BY_POS) == false )  continue;
      if ( OrderSymbol() != Symbol() || OrderMagicNumber() != MagicNum)  continue;
      
      if((OrderType() == OP_BUY || OrderType() == OP_BUYSTOP) && (OrderSymbol()==Symbol()))
          {
            
            if (BuyExit)
            {
   // try to close 3 Times
      
              CloseCnt = 0;
              while (CloseCnt < 3)
              {
                if (!OrderClose(OrderTicket(),OrderLots(),Bid,Slippage,Violet))
                {
                 err=GetLastError();
                 Print(CloseCnt," Error closing order : (", err , ") " + ErrorDescription(err));
                 if (err > 0) CloseCnt++;
                }
                else
                {
                 CloseCnt = 3;
                }
               }
               return (0);
             }
                      // check for stop
             if(Stop>0)  
             {                 
               if(Bid-OrderOpenPrice()>=Point*Stop)
               {
                   OrderClose(OrderTicket(),OrderLots(),Bid,3,Violet); // close position
                   return(0);
               }
             }
             else
             {
              if(TrailingStop>0) 
               {                
                 if(Bid-OrderOpenPrice()>Point*TrailingStop)
                 {
                  if(OrderStopLoss()<Bid-Point*TrailingStop)
                  {
                     CloseCnt=0;
                     while (CloseCnt < 3)
                     {
                     if (!OrderModify(OrderTicket(),OrderOpenPrice(),Bid-Point*TrailingStop,OrderTakeProfit(),0,Aqua))
                     {
                        err=GetLastError();
                        if (err>0) CloseCnt++;
                     }
                     else CloseCnt = 3;
                     }
                  }
                 }
              }
              return(0);
             }
          }

      if((OrderType() == OP_SELL || OrderType() == OP_SELLSTOP) && (OrderSymbol()==Symbol()))
          {
            if (SellExit)
            {
            
   // try to close 3 Times
      
               CloseCnt = 0;
               while (CloseCnt < 3)
               {
                 if (!OrderClose(OrderTicket(),OrderLots(),Ask,Slippage,Violet))
                 {
                  err=GetLastError();
                  Print(CloseCnt," Error closing order : ",cnt," (", err , ") " + ErrorDescription(err));
                  if (err > 0) CloseCnt++;
                 }
                 else CloseCnt = 3;
               }
               return(0);
             }
        // check for stop
             if(Stop>0)  
             {                 
                if(OrderOpenPrice()-Ask>=Point*Stop)
                {
                   OrderClose(OrderTicket(),OrderLots(),Ask,3,Violet); // close position
                   return(0);
                }
             }
           
   // try to modify 3 Times
      
             if(TrailingStop>0)  
             {                
                if((OrderOpenPrice()-Ask)>(Point*TrailingStop))
                {
                   if(OrderStopLoss()==0.0 || OrderStopLoss()>(Ask+Point*TrailingStop))
                   {
                     CloseCnt = 0;
                     while (CloseCnt < 3)
                     {
                       if (!OrderModify(OrderTicket(),OrderOpenPrice(),Ask+Point*TrailingStop,OrderTakeProfit(),0,Aqua))
                       {
                         err=GetLastError();
                         if (err > 0) CloseCnt++;
                       }
                       else CloseCnt = 3;
                     }
                   }
                 }
                 return(0);
              }
         }
    }
}


//+------------------------------------------------------------------+
//| Time frame interval appropriation  function                               |
//+------------------------------------------------------------------+

int func_TimeFrame_Const2Val(int Constant ) {
   switch(Constant) {
      case 1:  // M1
         return(1);
      case 5:  // M5
         return(2);
      case 15:
         return(3);
      case 30:
         return(4);
      case 60:
         return(5);
      case 240:
         return(6);
      case 1440:
         return(7);
      case 10080:
         return(8);
      case 43200:
         return(9);
   }
}

//+------------------------------------------------------------------+
//| Time frame string appropriation  function                               |
//+------------------------------------------------------------------+

string func_TimeFrame_Val2String(int Value ) {
   switch(Value) {
      case 1:  // M1
         return("PERIOD_M1");
      case 2:  // M1
         return("PERIOD_M5");
      case 3:
         return("PERIOD_M15");
      case 4:
         return("PERIOD_M30");
      case 5:
         return("PERIOD_H1");
      case 6:
         return("PERIOD_H4");
      case 7:
         return("PERIOD_D1");
      case 8:
         return("PERIOD_W1");
      case 9:
         return("PERIOD_MN1");
   	default: 
   		return("undefined " + Value);
   }
}

int func_Symbol2Val(string symbol) {
	if(symbol=="AUDUSD") {	return(01);

	} else if(symbol=="CHFJPY") {	return(10);

	} else if(symbol=="EURAUD") {	return(10);
	} else if(symbol=="EURCAD") {	return(11);
	} else if(symbol=="EURCHF") {	return(12);
	} else if(symbol=="EURGBP") {	return(13);
	} else if(symbol=="EURJPY") {	return(14);
	} else if(symbol=="EURUSD") {	return(15);

	} else if(symbol=="GBPCHF") {	return(20);
	} else if(symbol=="GBPJPY") {	return(21);
	} else if(symbol=="GBPUSD") { return(22);


	} else if(symbol=="USDCAD") {	return(40);
	} else if(symbol=="USDCHF") {	return(41);
	} else if(symbol=="USDJPY") {	return(42);


	} else if(symbol=="GOLD") {	return(90);
	} else {	Comment("unexpected Symbol"); return(0);
	}
}

//+------------------------------------------------------------------+
//| Average price efficiency  function                               |
//+------------------------------------------------------------------+

double MinOrderPrice(int OType, int OMagicNumber) {
double MinPrice;

   if (OrderType()==OP_BUY) {
      MinPrice=1000000;
   } else {
      MinPrice=0;
   }
 
   for(int cnt=0;cnt<OrdersTotal();cnt++)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      
      if(OrderType()==OType && OrderSymbol()==Symbol() && OrderMagicNumber()==OMagicNumber) {
         if (OrderType()==OP_BUY) {
            if (OrderOpenPrice()<MinPrice) {
               MinPrice=OrderOpenPrice();
            }
         } else {
            if (OrderOpenPrice()>MinPrice) {
               MinPrice=OrderOpenPrice();
            }
         }
      }
   }
   if (MinPrice==1000000) MinPrice=0;
   return(MinPrice);
   
}

int OrderSendExtended(string symbol, int cmd, double volume, double price, int slippage, double stoploss, double takeprofit, string comment, int magic, datetime expiration=0, color arrow_color=CLR_NONE) {
   datetime OldCurTime;
   int timeout=30;

   OldCurTime=CurTime();
   while (GlobalVariableCheck("InTrade") && !IsTradeAllowed()) {
      if(OldCurTime+timeout<=CurTime()) {
         Print("Error in OrderSendExtended(): Timeout encountered");
         return(0); 
      }
      Sleep(1000);
   }
     
   GlobalVariableSet("InTrade", CurTime());  // set lock indicator
   ticket = OrderSend(symbol, cmd, volume, price, slippage, stoploss, takeprofit, comment, magic, expiration, arrow_color);
   GlobalVariableDel("InTrade");   // clear lock indicator
   return(ticket);
}